// Type definitions for pure-react-carousel 1.12.0
// Definitions by: Jedrzej Lewandowski <https://github.com/TheFullResolution>
// TypeScript Version: 2.7.2

import * as React from 'react'
import {
  ButtonBack,
  ButtonFirst,
  ButtonLast,
  ButtonNext,
  ButtonPlay,
  Dot,
  DotGroup,
  Image,
  ImageWithZoom,
  Slide,
  Slider,
  SliderProps,
  SlideProps,
  ImageWithZoomProps,
  ImageProps,
  DotGroupProps,
  DotProps,
  ButtonBackProps,
  ButtonNextProps,
  ButtonLastProps,
  ButtonFirstProps,
  ButtonPlayProps
} from './carouselElements.d'

interface CarouselState {
  readonly currentSlide: number
  readonly disableAnimation: boolean
  readonly disableKeyboard: boolean
  readonly hasMasterSpinner: boolean
  readonly imageErrorCount: number
  readonly imageSuccessCount: number
  readonly lockOnWindowScroll: boolean
  readonly masterSpinnerFinished: boolean
  readonly masterSpinnerThreshold: number
  readonly naturalSlideHeight: number
  readonly naturalSlideWidth: number
  readonly orientation: 'horizontal' | 'vertical'
  readonly slideSize: number
  readonly slideTraySize: number
  readonly step: number
  readonly dragStep: number
  readonly totalSlides: number
  readonly touchEnabled: boolean
  readonly dragEnabled: boolean
  readonly visibleSlides: number
  readonly infinite: boolean
}

interface CarouselProviderProps {
  readonly children: React.ReactNode
  readonly className?: string
  readonly currentSlide?: CarouselState['currentSlide']
  readonly disableAnimation?: CarouselState['disableAnimation']
  readonly disableKeyboard?: CarouselState['disableKeyboard']
  readonly hasMasterSpinner?: CarouselState['hasMasterSpinner']
  readonly interval?: number
  readonly isPlaying?: boolean
  readonly lockOnWindowScroll?: CarouselState['lockOnWindowScroll']
  readonly naturalSlideHeight: CarouselState['naturalSlideHeight']
  readonly naturalSlideWidth: CarouselState['naturalSlideWidth']
  readonly playDirection?: 'forward'|'backward'
  readonly orientation?: CarouselState['orientation']
  readonly step?: CarouselState['step']
  readonly dragStep?: CarouselState['dragStep']
  readonly tag?: string
  readonly totalSlides: CarouselState['totalSlides']
  readonly touchEnabled?: CarouselState['touchEnabled']
  readonly dragEnabled?: CarouselState['dragEnabled']
  readonly visibleSlides?: CarouselState['visibleSlides']
  readonly infinite?: CarouselState['infinite']
}

type CarouselProviderInterface = React.ComponentClass<CarouselProviderProps>
/**
 * CarouselProvider allows the other carousel components to communicate with each other.
 *
 * The only required properties are:
 * the totalSlides, naturalSlideWidth, and naturalSlideHeight.
 *
 * The naturalSlideWidth and naturalSlideHeight are used
 * to create an aspect ratio for each slide.
 *
 * Since the carousel is responsive by default,
 * it will stretch to fill in the width of it's parent container.
 *
 * The CarouselProvider must also have children.
 */
declare const CarouselProvider: CarouselProviderInterface

export interface CarouselInjectedProps {
  readonly carouselStore: {
    readonly setStoreState: (state: Partial<CarouselState>) => void
    readonly unsubscribeAllMasterSpinner: () => void
  }
}

// Diff / Omit taken from https://github.com/Microsoft/TypeScript/issues/12215#issuecomment-311923766
type Diff<T extends string, U extends string> = ({ [P in T]: P } &
  { [P in U]: never } & { readonly [x: string]: never })[T]
type Omit<T, K extends keyof T> = Pick<T, Diff<Extract<keyof T, string>, Extract<K, string>>>

type MapStateToProps<TStateProps> = (state: CarouselState) => TStateProps

interface WithStoreInterface {
  <CustomProps extends CarouselInjectedProps>(
    component: React.ComponentClass<CustomProps>
  ): React.ComponentClass<Omit<CustomProps, keyof CarouselInjectedProps>> & {
    readonly WrappedComponent: React.ComponentClass<CustomProps>
  }

  <CustomProps extends CarouselInjectedProps, CustomStateProps>(
    component: React.ComponentClass<CustomProps & CustomStateProps>,
    state: MapStateToProps<CustomStateProps>
  ): React.ComponentClass<Omit<CustomProps, keyof CarouselInjectedProps>> & {
    readonly WrappedComponent: React.ComponentClass<
      CustomProps & CustomStateProps
    >
  }
}
/**
 * Use this HOC to pass CarouselProvider state properties as props to a component.
 * Basically, Your custom component must be an descendant of <CarouselProvider>.
 * It doesn't have to be a direct descendant,
 * it just needs to be between some the opening and closing CarouselProvider tags somewhere.
 *
 * <CarouselProvider>
 *  <YourComponentHere />
 * </CarouselProvider>
 *
 * WithStore has two arguments:
 *  WithStore([component], [mapstateToProps])
 *
 *  The first argument is the component to wrap (ex: YourComponentHere) and it's required.
 *
 *  The second argument is optional.
 *  It is a "map state to props" function that you must create.
 *
 */
declare const WithStore: WithStoreInterface

export {
  ButtonBack,
  ButtonFirst,
  ButtonLast,
  ButtonNext,
  ButtonPlay,
  CarouselProvider,
  CarouselProviderProps,
  Dot,
  DotGroup,
  Image,
  ImageWithZoom,
  Slide,
  Slider,
  WithStore,
  SliderProps,
  SlideProps,
  ImageWithZoomProps,
  ImageProps,
  DotGroupProps,
  DotProps,
  ButtonBackProps,
  ButtonNextProps,
  ButtonLastProps,
  ButtonFirstProps,
  ButtonPlayProps
}
